@@ -9,8 +9,8 @@ from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union
 
 from ccimport import compat, loader
 
-from pccm.constants import (PCCM_FUNC_META_KEY, PCCM_INIT_DECORATOR_KEY,
-                            PCCM_MAGIC_STRING)
+from pccm.constants import (PCCM_CLASS_META_KEY, PCCM_FUNC_META_KEY,
+                            PCCM_INIT_DECORATOR_KEY)
 from pccm.core.buildmeta import BuildMeta, _unique_list_keep_order
 from pccm.core.codegen import Block, generate_code, generate_code_list
 
@@ -34,7 +34,8 @@ class FunctionMeta(object):
                  name: Optional[str] = None,
                  macro_guard: Optional[str] = None,
                  impl_loc: str = "",
-                 impl_file_suffix: str = ".cc"):
+                 impl_file_suffix: str = ".cc",
+                 header_only: Optional[bool] = None):
         self.name = name
         if attrs is None:
             attrs = []
@@ -51,6 +52,7 @@ class FunctionMeta(object):
         self.virtual = virtual
         self.pure_virtual = pure_virtual
         self.macro_guard = macro_guard
+        self.header_only_user = header_only
 
     def get_pre_attrs(self) -> List[str]:
         res = self.attrs.copy()  # type: List[str]
@@ -64,7 +66,16 @@ class FunctionMeta(object):
         return []
 
     def is_header_only(self):
-        return self.inline or self.constexpr or self.pure_virtual
+        if self.header_only_user is not None:
+            return self.inline or self.constexpr or self.pure_virtual or self.header_only_user
+        else:
+            return self.inline or self.constexpr or self.pure_virtual
+
+
+class ClassMeta(object):
+    def __init__(self, name: Optional[str] = None, skip_inherit: bool = False):
+        self.skip_inherit = skip_inherit
+        self.name = name
 
 
 def get_func_meta_except(func) -> FunctionMeta:
@@ -74,6 +85,12 @@ def get_func_meta_except(func) -> FunctionMeta:
     return getattr(func, PCCM_FUNC_META_KEY)
 
 
+def get_class_meta(cls: Type) -> Optional[ClassMeta]:
+    if not hasattr(cls, PCCM_CLASS_META_KEY):
+        return None
+    return getattr(cls, PCCM_CLASS_META_KEY)
+
+
 class ConstructorMeta(FunctionMeta):
     def __init__(self,
                  inline: bool = False,
@@ -83,14 +100,16 @@ class ConstructorMeta(FunctionMeta):
                  name: Optional[str] = None,
                  macro_guard: Optional[str] = None,
                  impl_loc: str = "",
-                 impl_file_suffix: str = ".cc"):
+                 impl_file_suffix: str = ".cc",
+                 header_only: Optional[bool] = None):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          attrs=attrs,
                          name=name,
                          macro_guard=macro_guard,
                          impl_loc=impl_loc,
-                         impl_file_suffix=impl_file_suffix)
+                         impl_file_suffix=impl_file_suffix,
+                         header_only=header_only)
         self.explicit = explicit
 
     def get_pre_attrs(self) -> List[str]:
@@ -112,7 +131,8 @@ class DestructorMeta(FunctionMeta):
                  name: Optional[str] = None,
                  macro_guard: Optional[str] = None,
                  impl_loc: str = "",
-                 impl_file_suffix: str = ".cc"):
+                 impl_file_suffix: str = ".cc",
+                 header_only: Optional[bool] = None):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          attrs=attrs,
@@ -121,7 +141,8 @@ class DestructorMeta(FunctionMeta):
                          name=name,
                          macro_guard=macro_guard,
                          impl_loc=impl_loc,
-                         impl_file_suffix=impl_file_suffix)
+                         impl_file_suffix=impl_file_suffix,
+                         header_only=header_only)
         self.override = override
         self.final = final
         if override or final:
@@ -155,7 +176,8 @@ class MemberFunctionMeta(FunctionMeta):
                  name: Optional[str] = None,
                  macro_guard: Optional[str] = None,
                  impl_loc: str = "",
-                 impl_file_suffix: str = ".cc"):
+                 impl_file_suffix: str = ".cc",
+                 header_only: Optional[bool] = None):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          virtual=virtual,
@@ -164,7 +186,8 @@ class MemberFunctionMeta(FunctionMeta):
                          name=name,
                          macro_guard=macro_guard,
                          impl_loc=impl_loc,
-                         impl_file_suffix=impl_file_suffix)
+                         impl_file_suffix=impl_file_suffix,
+                         header_only=header_only)
         self.override = override
         self.final = final
         if override or final:
@@ -196,14 +219,16 @@ class StaticMemberFunctionMeta(FunctionMeta):
                  name: Optional[str] = None,
                  macro_guard: Optional[str] = None,
                  impl_loc: str = "",
-                 impl_file_suffix: str = ".cc"):
+                 impl_file_suffix: str = ".cc",
+                 header_only: Optional[bool] = None):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          attrs=attrs,
                          name=name,
                          macro_guard=macro_guard,
                          impl_loc=impl_loc,
-                         impl_file_suffix=impl_file_suffix)
+                         impl_file_suffix=impl_file_suffix,
+                         header_only=header_only)
 
     def get_pre_attrs(self) -> List[str]:
         res = super().get_pre_attrs()  # type: List[str]
@@ -221,14 +246,16 @@ class ExternalFunctionMeta(FunctionMeta):
                  name: Optional[str] = None,
                  macro_guard: Optional[str] = None,
                  impl_loc: str = "",
-                 impl_file_suffix: str = ".cc"):
+                 impl_file_suffix: str = ".cc",
+                 header_only: Optional[bool] = None):
         super().__init__(inline=inline,
                          constexpr=constexpr,
                          attrs=attrs,
                          name=name,
                          macro_guard=macro_guard,
                          impl_loc=impl_loc,
-                         impl_file_suffix=impl_file_suffix)
+                         impl_file_suffix=impl_file_suffix,
+                         header_only=header_only)
 
 
 class Argument(object):
@@ -415,6 +442,8 @@ class FunctionCode(object):
         self.func_doc = None  # type: Optional[str]
         self.code_after_include = None  # type: Optional[str]
 
+        self._additional_pre_attrs = []  # type: List[str]
+
     def is_template(self) -> bool:
         return len(self._template_arguments) > 0
 
@@ -456,7 +485,7 @@ class FunctionCode(object):
             yield
 
     @contextlib.contextmanager
-    def range_(self, var: str, stop: str, prefix: str = ""):
+    def range_(self, var: str, stop: Union[str, int], prefix: str = ""):
         with self.for_(
                 "int {i} = 0; {i} < {stop}; ++{i}".format(i=var, stop=stop),
                 prefix):
@@ -530,7 +559,8 @@ class FunctionCode(object):
         pre_attrs ret_type name(args) post_attrs;
         """
         header_only = meta.is_header_only() or self.is_template()
-        pre_attrs = _unique_list_keep_order(meta.get_pre_attrs())
+        pre_attrs = _unique_list_keep_order(meta.get_pre_attrs() +
+                                            self._additional_pre_attrs)
         post_attrs = _unique_list_keep_order(meta.get_post_attrs())
         return_type = self.return_type
         if isinstance(meta, (ConstructorMeta, DestructorMeta)):
@@ -566,7 +596,10 @@ class FunctionCode(object):
         if pre_attrs_str:
             prefix_fmt = pre_attrs_str + " " + prefix_fmt
         doc = self.generate_cpp_doc()
-        return doc + "\n" + template_fmt + prefix_fmt
+        if meta.macro_guard is not None:
+            return doc + "\n" + "#if {}\n".format(meta.macro_guard) + template_fmt + prefix_fmt + "\n#endif"
+        else:
+            return doc + "\n" + template_fmt + prefix_fmt
 
     def get_impl(self, name: str, meta: FunctionMeta, class_name: str = ""):
         """
@@ -576,7 +609,8 @@ class FunctionCode(object):
         if meta.pure_virtual:
             return self.get_sig(name, meta)
         header_only = meta.is_header_only() or self.is_template()
-        pre_attrs = _unique_list_keep_order(meta.get_pre_attrs())
+        pre_attrs = _unique_list_keep_order(meta.get_pre_attrs() +
+                                            self._additional_pre_attrs)
         post_attrs = _unique_list_keep_order(meta.get_post_attrs())
         if not header_only:
             pre_attrs = self._clean_pre_attrs_impl(pre_attrs)
@@ -619,6 +653,8 @@ class FunctionCode(object):
         blocks = []  # List[Union[Block, str]]
         blocks.extend(self._blocks)
         block = Block(template_fmt + prefix_fmt, blocks, "}")
+        if meta.macro_guard is not None:
+            block = Block("#if {}".format(meta.macro_guard), [block], "#endif")
         return block
 
     def arg(self,
@@ -722,6 +758,9 @@ class FunctionCode(object):
         fmts.append(" */")
         return "\n".join(fmts)
 
+    def add_pre_attr(self, attr: str):
+        self._additional_pre_attrs.append(attr)
+
 
 class FunctionDecl(object):
     def __init__(self, meta: FunctionMeta, code: FunctionCode):
@@ -766,6 +805,8 @@ class Class(object):
         only works in python 3.6+, so we don't support inherit
         in python 3.5.
         """
+        if hasattr(cls, PCCM_CLASS_META_KEY):
+            setattr(cls, PCCM_CLASS_META_KEY, None)
         cls.__init__ = _init_decorator(cls.__init__, cls)
         return super().__init_subclass__()
 
@@ -784,14 +825,12 @@ class Class(object):
         }  # type: Dict[Optional[Type[Class]], List[Member]]
 
         self._param_class = OrderedDict(
-        )  # type: OrderedDict[str, ParameterizedClass]
+        )  # type: OrderedDict[str, List[Tuple[ParameterizedClass, Optional[str]]]]
 
         self._this_type_to_param_class = {
             self._this_cls_type: OrderedDict()
         }  # type: Dict[Optional[Type[Class]], Dict[str, ParameterizedClass]]
 
-        self._param_class_alias = OrderedDict()  # type: OrderedDict[str, str]
-
         self._this_type_to_param_class_alias = {
             self._this_cls_type: OrderedDict()
         }  # type: Dict[Optional[Type[Class]], Dict[str, str]]
@@ -962,26 +1001,30 @@ class Class(object):
             msg = "can only add Param Class, but your {} is Class".format(
                 param_class.class_name)
             raise ValueError(msg)
-        assert subnamespace not in self._param_class
-        self._param_class[subnamespace] = param_class
-        if name_alias is not None:
-            self._param_class_alias[subnamespace] = name_alias
+        if subnamespace not in self._param_class:
+            self._param_class[subnamespace] = []
+        self._param_class[subnamespace].append((param_class, name_alias))
 
-    def add_impl_only_dependency(self, func,
+    def add_impl_only_dependency(self,
+                                 func_or_list_funcs: Union[Callable,
+                                                           List[Callable]],
                                  *no_param_class_cls: Type["Class"]):
-        if inspect.ismethod(func):
-            func = func.__func__
-        func_meta = get_func_meta_except(func)
-        if func_meta.inline:
-            raise ValueError("inline function can't have impl-only dep")
-        name = func.__name__
-        if name not in self._impl_only_cls_dep:
-            self._impl_only_cls_dep[name] = []
-        for npcls in no_param_class_cls:
-            self._impl_only_cls_dep[name].append(npcls)
-            if npcls not in self._deps:
-                # only add once
-                self.add_dependency(npcls)
+        if not isinstance(func_or_list_funcs, list):
+            func_or_list_funcs = [func_or_list_funcs]
+        for func in func_or_list_funcs:
+            if inspect.ismethod(func):
+                func = func.__func__
+            func_meta = get_func_meta_except(func)
+            if func_meta.inline:
+                raise ValueError("inline function can't have impl-only dep")
+            name = func.__name__
+            if name not in self._impl_only_cls_dep:
+                self._impl_only_cls_dep[name] = []
+            for npcls in no_param_class_cls:
+                self._impl_only_cls_dep[name].append(npcls)
+                if npcls not in self._deps:
+                    # only add once
+                    self.add_dependency(npcls)
 
     def add_impl_only_param_class(self,
                                   func_or_list_funcs: Union[Callable,
@@ -1097,9 +1140,18 @@ class Class(object):
         if type(self) is Class:
             return None
         pccm_base_types = []  # List[Type[Class]]
-        for base in type(self).__bases__:
+        candidates = list(type(self).__bases__)
+        while candidates:
+            base = candidates.pop()
             if issubclass(base, Class):
-                pccm_base_types.append(base)
+                cls_meta = get_class_meta(base)
+                if cls_meta is None:
+                    pccm_base_types.append(base)
+                else:
+                    if cls_meta.skip_inherit:
+                        candidates.extend(base.__bases__)
+                    else:
+                        pccm_base_types.append(base)
         assert len(pccm_base_types) == 1, "you can only inherit one class."
         pccm_base = pccm_base_types[0]
         if pccm_base is not Class and base is not ParameterizedClass:
@@ -1134,12 +1186,13 @@ class Class(object):
             ns_stmt = "using {} = {};".format(dep.class_name, name_with_ns)
             return ns_stmt
         else:
-            for k, v in self._param_class.items():
-                if dep is v and k in self._param_class_alias:
-                    name_with_ns = "::".join(dep.namespace.split("."))
-                    ns_stmt = "using {} = {}::{};".format(
-                        self._param_class_alias[k], name_with_ns, v.class_name)
-                    return ns_stmt
+            for k, pcls_alias_tuples in self._param_class.items():
+                for pcls, alias in pcls_alias_tuples:
+                    if dep is pcls and alias is not None:
+                        name_with_ns = "::".join(dep.namespace.split("."))
+                        ns_stmt = "using {} = {}::{};".format(
+                            alias, name_with_ns, pcls.class_name)
+                        return ns_stmt
         return None
 
     def get_dependency_aliases(self) -> List[str]:
@@ -1519,6 +1572,8 @@ class CodeGenerator(object):
         3. Class must be unique.
         """
         # assert self.built is False
+        if not cus:
+            return []
         for cu in cus:
             if isinstance(cu, ParameterizedClass):
                 assert cu.namespace is not None
@@ -1562,15 +1617,17 @@ class CodeGenerator(object):
                         cur_type_trace_copy.add(dep)
                         stack.append((cu_type_to_cu[dep], cur_type_trace_copy))
 
-                    for k, v in cur_cu._param_class.items():
-                        if type(v) in cur_type_trace:
-                            raise ValueError("cycle detected")
-                        if v.namespace is None:
-                            v.namespace = cur_ns + "." + k
-                        cur_cu._unified_deps.append(v)
-                        cur_type_trace_copy = cur_type_trace.copy()
-                        cur_type_trace_copy.add(type(v))
-                        stack.append((v, cur_type_trace_copy))
+                    for k, pcls_with_alias_tuples in cur_cu._param_class.items(
+                    ):
+                        for pcls, _ in pcls_with_alias_tuples:
+                            if type(pcls) in cur_type_trace:
+                                raise ValueError("cycle detected")
+                            if pcls.namespace is None:
+                                pcls.namespace = cur_ns + "." + k
+                            cur_cu._unified_deps.append(pcls)
+                            cur_type_trace_copy = cur_type_trace.copy()
+                            cur_type_trace_copy.add(type(pcls))
+                            stack.append((pcls, cur_type_trace_copy))
                     cur_cu._function_decls.extend(
                         self.cached_extract_classunit_methods(cur_cu))
                     cur_cu._assign_overload_flag_to_func_decls()
@@ -1759,6 +1816,7 @@ class CodeGenerator(object):
                             for cls_dep in cls_deps:
                                 if udep_type is cls_dep:
                                     impl_only_deps[impl_file_name].append(udep)
+
             for impl_func_name, pcls_deps in cu._impl_only_param_cls_dep.items(
             ):
                 if impl_func_name == meta.name:
