@@ -1,7 +1,7 @@
 import ast
 from collections import OrderedDict
 from enum import Enum
-from typing import Callable, Dict, List, Optional, Set, Union
+from typing import Callable, Dict, List, Optional, Set, Tuple, Union
 
 from ccimport import compat
 
@@ -14,28 +14,6 @@ from pccm.core.buildmeta import _unique_list_keep_order
 from pccm.core.codegen import Block, generate_code, generate_code_list
 from pccm.core.markers import middleware_decorator
 
-_AUTO_ANNO_TYPES = {
-    "int": "int",
-    "int8_t": "int",
-    "int16_t": "int",
-    "int32_t": "int",
-    "int64_t": "int",
-    "uint8_t": "int",
-    "uint16_t": "int",
-    "uint32_t": "int",
-    "uint64_t": "int",
-    "unsigned": "int",
-    "long": "int",
-    "short": "int",
-    "float": "float",
-    "double": "float",
-    "unsigned long": "int",
-    "unsigned int": "int",
-    "bool": "bool",
-    "std::string": "str",
-    "void": "None",
-}  # type: Dict[str, str]
-
 _IDENTITY_DEFAULT_HANDLER = lambda x: x
 
 
@@ -87,36 +65,41 @@ def get_attribute_name_parts(node):
     return parts[::-1]
 
 
-def _anno_parser(node: ast.AST, imports: List[str]) -> str:
+def _anno_parser(node: ast.AST, imports: List[str], disable_from_import: bool) -> str:
     if isinstance(node, ast.Module):
         assert len(node.body) == 1
-        return _anno_parser(node.body[0], imports)
+        return _anno_parser(node.body[0], imports, disable_from_import)
     if isinstance(node, ast.Expr):
-        return _anno_parser(node.value, imports)
+        return _anno_parser(node.value, imports, disable_from_import)
     elif isinstance(node, ast.Subscript):
         assert isinstance(node.value, (ast.Name, ast.Attribute))
         if compat.Python3_9AndLater:
             # >= 3.9 ast.Index is deprecated
             assert isinstance(node.slice, ast.Tuple)
-            return "{}[{}]".format(_anno_parser(node.value, imports),
-                                   _anno_parser(node.slice, imports))
+            return "{}[{}]".format(_anno_parser(node.value, imports, disable_from_import),
+                                   _anno_parser(node.slice, imports, disable_from_import))
         else:
             assert isinstance(node.slice, ast.Index)
             assert isinstance(
                 node.slice.value,
                 (ast.Tuple, ast.Attribute, ast.Name, ast.Subscript))
-            return "{}[{}]".format(_anno_parser(node.value, imports),
-                                   _anno_parser(node.slice.value, imports))
+            return "{}[{}]".format(_anno_parser(node.value, imports, disable_from_import),
+                                   _anno_parser(node.slice.value, imports, disable_from_import))
     elif isinstance(node, ast.Tuple):
-        return ", ".join(_anno_parser(e, imports) for e in node.elts)
+        return ", ".join(_anno_parser(e, imports, disable_from_import) for e in node.elts)
     elif isinstance(node, ast.Name):
         return node.id
     elif isinstance(node, ast.Attribute):
         value_parts = get_attribute_name_parts(node)  # type: List[str]
-        from_import = "from {} import {}".format(".".join(value_parts[:-1]),
-                                                 value_parts[-1])
-        imports.append(from_import)
-        return value_parts[-1]
+        if disable_from_import:
+            regular_import = "import {}".format(value_parts[0])
+            imports.append(regular_import)
+            return ".".join(value_parts)
+        else:
+            from_import = "from {} import {}".format(".".join(value_parts[:-1]),
+                                                    value_parts[-1])
+            imports.append(from_import)
+            return value_parts[-1]
     else:
         msg = "pyanno only support format like name.attr[nested_name.attr[name1, name2], name3]\n"
         msg += "but we get ast node {}".format(ast.dump(node))
@@ -124,24 +107,174 @@ def _anno_parser(node: ast.AST, imports: List[str]) -> str:
 
 
 class TemplateTypeStmt(object):
-    def __init__(self, name: str, args: List[Union[str, "TemplateTypeStmt"]]):
+    NameToHandler = {
+        "int":
+        lambda args: "int",
+        "int8_t":
+        lambda args: "int",
+        "int16_t":
+        lambda args: "int",
+        "int32_t":
+        lambda args: "int",
+        "int64_t":
+        lambda args: "int",
+        "uint8_t":
+        lambda args: "int",
+        "uint16_t":
+        lambda args: "int",
+        "uint32_t":
+        lambda args: "int",
+        "uint64_t":
+        lambda args: "int",
+        "unsigned":
+        lambda args: "int",
+        "long":
+        lambda args: "int",
+        "short":
+        lambda args: "int",
+        "float":
+        lambda args: "float",
+        "double":
+        lambda args: "float",
+        "unsigned long":
+        lambda args: "int",
+        "unsigned int":
+        lambda args: "int",
+        "unsigned long long":
+        lambda args: "int",
+        "bool":
+        lambda args: "bool",
+        "std::string":
+        lambda args: "str",
+        "void":
+        lambda args: "None",
+        "std::array":
+        lambda args: "List[{}]".format(args[0].to_pyanno()),
+        "tv::array":
+        lambda args: "List[{}]".format(args[0].to_pyanno()),
+        "std::tuple":
+        lambda args: "Tuple[{}]".format(", ".join(a.to_pyanno()
+                                                  for a in args)),
+        "std::vector":
+        lambda args: "List[{}]".format(args[0].to_pyanno()),
+        "std::list":
+        lambda args: "List[{}]".format(args[0].to_pyanno()),
+        "std::map":
+        lambda args: "Dict[{}]".format(", ".join(a.to_pyanno()
+                                                 for a in args[:2])),
+        "std::unordered_map":
+        lambda args: "Dict[{}]".format(", ".join(a.to_pyanno()
+                                                 for a in args[:2])),
+        "std::set":
+        lambda args: "Set[{}]".format(args[0].to_pyanno()),
+        "std::unordered_set":
+        lambda args: "Set[{}]".format(args[0].to_pyanno()),
+    }  # type: Dict[str, Callable[[List["TemplateTypeStmt"]], str]]
+
+    def __init__(self,
+                 name: str,
+                 args: List["TemplateTypeStmt"],
+                 not_template: bool,
+                 invalid: bool = False,
+                 exist_anno: str = ""):
         self.name = name
         self.args = args
+        self.not_template = not_template
+        self.invalid = invalid
+        self.exist_anno = exist_anno
+
+    def to_pyanno(self) -> str:
+        if self.invalid:
+            return "Any"
+        if self.exist_anno:
+            return self.exist_anno
+        if self.name in self.NameToHandler:
+            pyanno_generic = self.NameToHandler[self.name](self.args)
+        else:
+            pyanno_generic = "Any"
+        return pyanno_generic
 
 
-def _simple_template_type_parser(stmt: str, start: int, end: int):
-    # find '<'
-    assert stmt[end - 1] == ">"
-    left = -1
-    for i in range(start, end):
-        pass
-    pass
+def _simple_template_type_parser_recursive(
+        stmt: str, begin: int, end: int, bracket_pair: Dict[int, int],
+        exist_annos: Dict[str, str]) -> TemplateTypeStmt:
+    if stmt[end - 1] != ">":
+        # type with no template param
+        name = stmt[begin:end].strip()
+        if name in exist_annos:
+            return TemplateTypeStmt("", [], True, exist_anno=exist_annos[name])
+        return TemplateTypeStmt(name, [], True)
+    left = stmt.find("<", begin, end)
+
+    if left == -1:
+        raise ValueError("invalid")
+    name = stmt[begin:left].strip()
+    if name in exist_annos:
+        return TemplateTypeStmt("", [], True, exist_anno=exist_annos[name])
+    template_arg_ranges = []  # type: List[Tuple[int, int]]
+    pos = left + 1
+    arg_range_start = left + 1
+    while pos < end - 1:
+        if pos in bracket_pair:
+            pos = bracket_pair[pos] + 1
+        if pos >= end - 1:
+            break
+        val = stmt[pos]
+        if val == ",":
+            template_arg_ranges.append((arg_range_start, pos))
+            arg_range_start = pos + 1
+        pos += 1
+    template_arg_ranges.append((arg_range_start, end - 1))
+    args = [
+        _simple_template_type_parser_recursive(stmt, b, e, bracket_pair,
+                                               exist_annos)
+        for b, e in template_arg_ranges
+    ]
+    return TemplateTypeStmt(name, args, False)
+
+
+def _simple_template_type_parser(
+        stmt: str, exist_annos: Dict[str, str]) -> TemplateTypeStmt:
+    # TODO parse const/ref
+    invalid = TemplateTypeStmt("", [], False, True)
+    bracket_stack = []  # type: List[Tuple[str, int]]
+    N = len(stmt)
+    pos = 0
+    bracket_pair = {}  # type: Dict[int, int]
+    while pos < N:
+        val = stmt[pos]
+        if val == "<":
+            bracket_stack.append((val, pos))
+        elif val == ">":
+            if not bracket_stack:
+                return invalid
+            start_val, start = bracket_stack.pop()
+            bracket_pair[start] = pos
+        pos += 1
+    if bracket_stack:
+        return invalid
+    if "\"" in stmt:
+        res = TemplateTypeStmt("", [], False, True)
+    try:
+        res = _simple_template_type_parser_recursive(stmt, 0, len(stmt),
+                                                     bracket_pair, exist_annos)
+    except ValueError:
+        res = TemplateTypeStmt("", [], False, True)
+    return res
 
 
 def python_anno_parser(anno_str: str):
+    anno_str = anno_str.strip()
+    disable_from_import = False
+    if anno_str[0] == "~":
+        disable_from_import = True
+        anno_str = anno_str[1:]
+
+    if anno_str == "None":
+        return "None", []
     tree = ast.parse(anno_str)
     imports = []  # type: List[str]
-    refined_name = _anno_parser(tree, imports)
+    refined_name = _anno_parser(tree, imports, disable_from_import)
     return refined_name, imports
 
 
@@ -176,7 +309,8 @@ class Pybind11MethodMeta(Pybind11Meta):
                  prop_name: str = "",
                  ret_policy: ReturnPolicy = ReturnPolicy.Auto,
                  call_guard: Optional[str] = None,
-                 virtual: bool = False):
+                 virtual: bool = False,
+                 keep_alives: Optional[List[Tuple[int, int]]] = None):
         super().__init__(Pybind11SplitImpl)
         self.bind_name = bind_name
         self.method_type = method_type
@@ -184,14 +318,16 @@ class Pybind11MethodMeta(Pybind11Meta):
         self.ret_policy = ret_policy
         self.call_guard = call_guard
         self.virtual = virtual
+        self.keep_alives = keep_alives
 
 
 class Pybind11PropMeta(Pybind11Meta):
     """may add some attributes in future.
     """
-    def __init__(self, readwrite: bool = True):
+    def __init__(self, name: str, readwrite: bool = True):
         super().__init__(Pybind11SplitImpl)
         self.readwrite = readwrite
+        self.name = name 
 
 
 class PybindMethodDecl(object):
@@ -199,10 +335,10 @@ class PybindMethodDecl(object):
                  mw_meta: Pybind11MethodMeta):
         self.decl = decl
         self.mw_meta = mw_meta
+        self.func_name = decl.get_function_name()
+        self.bind_name = self.func_name
         if mw_meta.bind_name:
-            self.func_name = mw_meta.bind_name
-        else:
-            self.func_name = decl.get_function_name()
+            self.bind_name = mw_meta.bind_name
         self.method_type = mw_meta.method_type
         member_meta = decl.meta
         if mw_meta.virtual:
@@ -250,19 +386,26 @@ class PybindMethodDecl(object):
 
     def to_string(self) -> str:
         if isinstance(self.decl.meta, ConstructorMeta):
-            return ".def({}, {})".format(self.addr, ", ".join(self.args))
+            if self.args:
+                return ".def({}, {})".format(self.addr, ", ".join(self.args))
+            else:
+                return ".def({})".format(self.addr)
+
         def_stmt = "def"
-        func_name = self.func_name
+        bind_name = self.bind_name
 
         if isinstance(self.decl.meta, StaticMemberFunctionMeta):
             def_stmt = "def_static"
         if self.method_type == MethodType.PropGetter:
             def_stmt = "def_property_readonly"
-            func_name = self.mw_meta.prop_name
+            bind_name = self.mw_meta.prop_name
         addr = self.addr
         if self.decl.is_overload:
             addr = self.get_overload_addr()
         attrs = self.args.copy()
+        if self.mw_meta.keep_alives is not None:
+            attrs.extend("pybind11::keep_alive<{}, {}>()".format(x, y)
+                         for x, y in self.mw_meta.keep_alives)
         attrs.append(self.mw_meta.ret_policy.value)
         if self.mw_meta.call_guard is not None:
             attrs.append("pybind11::call_guard<{}>()".format(
@@ -272,10 +415,10 @@ class PybindMethodDecl(object):
             def_stmt = "def_property"
         # if self.decl.meta.mw_metas
         if attrs:
-            return ".{}(\"{}\", {}, {})".format(def_stmt, func_name, addr,
+            return ".{}(\"{}\", {}, {})".format(def_stmt, bind_name, addr,
                                                 ", ".join(attrs))
         else:
-            return ".{}(\"{}\", {})".format(def_stmt, func_name, addr)
+            return ".{}(\"{}\", {})".format(def_stmt, bind_name, addr)
 
     def get_virtual_string(self, parent_cls_name: str):
         fmt = "{} {} {{PYBIND11_OVERRIDE({}, {}, {}, {});}}"
@@ -292,7 +435,7 @@ class PybindMethodDecl(object):
                                          with_pure=False)
         arg_names = ", ".join(a.name for a in self.decl.code.arguments)
         return fmt.format(sig_str, override, self.decl.code.return_type,
-                          parent_cls_name, self.func_name, arg_names)
+                          parent_cls_name, self.bind_name, arg_names)
 
 
 class PybindPropDecl(object):
@@ -303,11 +446,17 @@ class PybindPropDecl(object):
         self.addr = "&{}::{}::{}".format(namespace.replace(".", "::"),
                                          class_name, self.decl.name)
 
+    def get_prop_name(self) -> str:
+        if self.mw_meta.name:
+            return self.mw_meta.name
+        else:
+            return self.decl.name
+
     def to_string(self) -> str:
         def_stmt = "def_readwrite"
         if not self.mw_meta.readwrite:
             def_stmt = "def_readonly"
-        return ".{}(\"{}\", {})".format(def_stmt, self.decl.name, self.addr)
+        return ".{}(\"{}\", {})".format(def_stmt, self.mw_meta.name, self.addr)
 
 
 class PybindClassMixin:
@@ -318,10 +467,14 @@ class PybindClassMixin:
                           array: Optional[str] = None,
                           pyanno: Optional[str] = None,
                           readwrite: bool = True,
+                          prop_name: Optional[str] = None,
                           mw_metas: Optional[List[MiddlewareMeta]] = None):
         if mw_metas is None:
             mw_metas = []
-        mw_metas.append(Pybind11PropMeta(readwrite))
+        if prop_name is None:
+            prop_name = name 
+        for prop_name_part in prop_name.split("."):
+            mw_metas.append(Pybind11PropMeta(prop_name_part, readwrite))
         return self.add_member(name=name,
                                type=type,
                                default=default,
@@ -437,20 +590,27 @@ def _postprocess_class(cls_name: str, cls_namespace: str, submod: str,
     return cls_def_block, vblock
 
 
-def _extract_anno_default(user_anno: Optional[str],
-                          type_str: str,
-                          cpp_default: Optional[str] = None):
+def _extract_anno_default(
+    user_anno: Optional[str],
+    type_str: str,
+    exist_annos: Dict[str, str],
+    cpp_default: Optional[str] = None,
+):
     from_imports = []  # type: List[str]
     default = None
     if user_anno is None:
-        if type_str in _AUTO_ANNO_TYPES:
-            user_anno = _AUTO_ANNO_TYPES[type_str]
+        try_extract_pyanno_res = _simple_template_type_parser(
+            type_str, exist_annos)
+        try_extract_pyanno = try_extract_pyanno_res.to_pyanno()
+        if try_extract_pyanno != "Any":
             if default is None:
                 if cpp_default is not None:
                     if type_str in _AUTO_ANNO_TYPES_DEFAULT_HANDLER:
                         handler = _AUTO_ANNO_TYPES_DEFAULT_HANDLER[type_str]
                         default = handler(cpp_default)
-            return user_anno, from_imports, default
+            try_extract_pyanno, from_imports = python_anno_parser(
+                try_extract_pyanno)
+            return try_extract_pyanno, from_imports, default
     anno = None
     if user_anno is not None:
         user_anno_type_default = user_anno.split("=")
@@ -468,10 +628,50 @@ def _extract_anno_default(user_anno: Optional[str],
     return anno, from_imports, default
 
 
+def _collect_exist_annos(decls: List[Union[PybindMethodDecl, PybindPropDecl]]):
+    # TODO handle annos for pccm classes
+    # we need to remove all annotations of pccm class
+
+    # split decl to method decl and prop decl
+    method_decls = []  # type: List[PybindMethodDecl]
+    prop_decls = []  # type: List[PybindPropDecl]
+    for decl in decls:
+        if isinstance(decl, PybindMethodDecl):
+            method_decls.append(decl)
+        else:
+            prop_decls.append(decl)
+    exist_annos = {}  # type: Dict[str, str]
+    for prop_decl in prop_decls:
+        prop_type = prop_decl.decl.type_str
+        user_anno = prop_decl.decl.pyanno
+        if user_anno is not None:
+            user_anno_pair = user_anno.split("=")
+            user_anno_type = user_anno_pair[0].strip()
+            exist_annos[prop_type] = user_anno_type
+
+    for pydecl in method_decls:
+        user_anno = pydecl.decl.code.ret_pyanno
+        ret_type = pydecl.decl.code.return_type
+        if user_anno is not None:
+            user_anno_pair = user_anno.split("=")
+            user_anno_type = user_anno_pair[0].strip()
+            exist_annos[ret_type] = user_anno_type
+
+        for arg in pydecl.decl.code.arguments:
+            user_anno = arg.pyanno
+            if user_anno is not None:
+                user_anno_pair = user_anno.split("=")
+                user_anno_type = user_anno_pair[0].strip()
+                exist_annos[arg.type_str] = user_anno_type
+
+    return exist_annos
+
+
 def _generate_python_interface_class(cls_name: str,
                                      decls: List[Union[PybindMethodDecl,
                                                        PybindPropDecl]],
-                                     enum_classes: List[EnumClass]):
+                                     enum_classes: List[EnumClass],
+                                     exist_annos: Dict[str, str]):
     """
     dep_imports
     class xxx:
@@ -499,7 +699,7 @@ def _generate_python_interface_class(cls_name: str,
         prop_type = prop_decl.decl.type_str
         user_anno = prop_decl.decl.pyanno
         anno, from_imports, default = _extract_anno_default(
-            user_anno, prop_type)
+            user_anno, prop_type, exist_annos)
         if anno is not None:
             prop_anno = anno
         imports.extend(from_imports)
@@ -508,13 +708,13 @@ def _generate_python_interface_class(cls_name: str,
         default_str = ""
         if default is not None:
             default_str = " = {}".format(default)
-        decl_codes.append("{}: {}{}".format(prop_decl.decl.name, prop_anno,
+        decl_codes.append("{}: {}{}".format(prop_decl.get_prop_name(), prop_anno,
                                             default_str))
     for decl in method_decls:
-        if decl.func_name not in name_to_overloaded:
-            name_to_overloaded[decl.func_name] = []
-        name_to_overloaded[decl.func_name].append(decl)
-    for func_name, over_decls in name_to_overloaded.items():
+        if decl.bind_name not in name_to_overloaded:
+            name_to_overloaded[decl.bind_name] = []
+        name_to_overloaded[decl.bind_name].append(decl)
+    for bind_name, over_decls in name_to_overloaded.items():
         if len(over_decls) == 1:
             fmt = "{}def {}({}){}: {}..."
         else:
@@ -531,22 +731,22 @@ def _generate_python_interface_class(cls_name: str,
             user_anno = pydecl.decl.code.ret_pyanno
             ret_type = pydecl.decl.code.return_type
             anno, from_imports, default = _extract_anno_default(
-                user_anno, ret_type)
+                user_anno, ret_type, exist_annos)
             if anno is not None:
                 if anno == cls_name:
                     anno = "\"{}\"".format(anno)
                 res_anno = " -> {}".format(anno)
                 imports.extend(from_imports)
-            decl_func_name = func_name
+            decl_bind_name = bind_name
             if isinstance(pydecl.decl.meta, ConstructorMeta):
-                decl_func_name = "__init__"
+                decl_bind_name = "__init__"
 
             arg_names = []  # type: List[str]
             have_default = False  # type: bool
             for arg in pydecl.decl.code.arguments:
                 user_anno = arg.pyanno
                 user_anno, from_imports, default = _extract_anno_default(
-                    user_anno, arg.type_str, arg.default)
+                    user_anno, arg.type_str, exist_annos, arg.default)
                 default_str = ""
                 if default is not None:
                     default_str = " = {}".format(default)
@@ -555,7 +755,7 @@ def _generate_python_interface_class(cls_name: str,
                     if have_default:
                         msg = ("you must provide a python default anno value "
                                "for {} of {}. format: PythonType = Default")
-                        raise ValueError(msg.format(arg.name, decl_func_name))
+                        raise ValueError(msg.format(arg.name, decl_bind_name))
                 if user_anno is not None:
                     if user_anno == cls_name:
                         user_anno = "\"{}\"".format(user_anno)
@@ -574,13 +774,13 @@ def _generate_python_interface_class(cls_name: str,
                 decorator = "@staticmethod\n"
             if pydecl.method_type == MethodType.PropGetter:
                 decorator = "@property\n"
-                decl_func_name = pydecl.mw_meta.prop_name
+                decl_bind_name = pydecl.mw_meta.prop_name
             elif pydecl.method_type == MethodType.PropSetter:
                 decorator = "@{}.setter\n".format(pydecl.mw_meta.prop_name)
-                decl_func_name = pydecl.mw_meta.prop_name
+                decl_bind_name = pydecl.mw_meta.prop_name
 
             decl_codes.append(
-                fmt.format(decorator, decl_func_name, py_sig, res_anno, doc))
+                fmt.format(decorator, decl_bind_name, py_sig, res_anno, doc))
     # Class EnumName:
     for ec in enum_classes:
         ec_items = []  # type: List[Union[Block, str]]
@@ -730,7 +930,6 @@ class Pybind11SplitMain(ParameterizedClass):
             methods (overloaded methods)
         TODO handle c++ operators
         TODO better code
-        TODO auto generate STL annotations
         TODO insert docstring if exists
         """
         init_import = "from typing import overload, Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union"
@@ -739,7 +938,10 @@ class Pybind11SplitMain(ParameterizedClass):
         ns_to_interfaces = OrderedDict()  # type: Dict[str, List[Block]]
         ns_to_imports = OrderedDict()  # type: Dict[str, List[str]]
         ns_to_interface = OrderedDict()  # type: Dict[str, str]
-
+        exist_annos = {}  # type: Dict[str, str]
+        for bind_cu in bind_cus:
+            exist_annos.update(_collect_exist_annos(
+                bind_cu.get_pybind_decls()))
         for bind_cu in bind_cus:
             origin_cu = bind_cu.cu
             ns = origin_cu.namespace
@@ -749,17 +951,27 @@ class Pybind11SplitMain(ParameterizedClass):
                 ns_to_imports[ns] = []
             class_block, cls_imports = _generate_python_interface_class(
                 origin_cu.class_name, bind_cu.get_pybind_decls(),
-                origin_cu._enum_classes)
+                origin_cu._enum_classes, exist_annos)
             ns_to_imports[ns].extend(cls_imports)
             ns_to_interfaces[ns].append(class_block)
+        module_as_init = set()  # type: Set[str]
+        for k, interfaces in ns_to_interfaces.items():
+            k_prefix = ".".join(k.split(".")[:-1])
+            if k_prefix and k_prefix in ns_to_interfaces:
+                module_as_init.add(k_prefix)
 
         for k, interfaces in ns_to_interfaces.items():
+            k_file = k
+            # if this module have submodule, we need
+            # to use a module directory instead a single file.
+            if k in module_as_init:
+                k_file += ".__init__"
             imports = ns_to_imports[k]
             imports.insert(0, init_pccm_import)
             imports.insert(0, init_import)
 
             imports = _unique_list_keep_order(imports)
-            ns_to_interface[k] = "\n".join(
+            ns_to_interface[k_file] = "\n".join(
                 generate_code_list(imports + interfaces, 0, 4))
         return ns_to_interface
 
@@ -805,14 +1017,16 @@ def mark(func=None,
          method_type: MethodType = MethodType.Normal,
          ret_policy: ReturnPolicy = ReturnPolicy.Auto,
          virtual: bool = False,
-         nogil: bool = False):
+         nogil: bool = False,
+         keep_alives: Optional[List[Tuple[int, int]]] = None):
     if virtual:
         assert not nogil, "you can't release gil for python virtual function."
     call_guard = None  # type: Optional[str]
     if nogil:
         call_guard = "pybind11::gil_scoped_release"
     pybind_meta = Pybind11MethodMeta(bind_name, method_type, prop_name,
-                                     ret_policy, call_guard, virtual)
+                                     ret_policy, call_guard, virtual,
+                                     keep_alives)
     return middleware_decorator(func, pybind_meta)
 
 
@@ -844,3 +1058,8 @@ if __name__ == "__main__":
     # print(ast.parse)
     print(python_anno_parser("Tuple[Tuple[spconv.Tensor, int], float]"))
     # python_anno_parser("Tuple[Tuple[spconv.Tensor, int], float]")
+    print(
+        _simple_template_type_parser("std::vector<std::tuple<int, ArrayPtr>>",
+                                     {
+                                         "ArrayPtr": "ArrayPtr"
+                                     }).to_pyanno())
